/*
 * Quote Plugin
 *
 * Copyright (C) 2008, Anton Pomozov <pomozoff@mail.ru>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02111-1301, USA.
 *
 */

#include "internal.h"
#include "version.h"
#include "pidginstock.h"
#include "gtkutils.h"
#include "gtkimhtml.h"
#include "debug.h"

#define PLUGIN_ID "gtk-convquote"
#define QUOTE_VERSION "0.9.0"

/* config.h may define PURPLE_PLUGINS; protect the definition here so that we
 * don't get complaints about redefinition when it's not necessary. */
#ifndef PURPLE_PLUGINS
# define PURPLE_PLUGINS
#endif
/* This will prevent compiler errors in some instances and is better explained in the
 * how-to documents on the wiki */
#ifndef G_GNUC_NULL_TERMINATED
# if __GNUC__ >= 4
#  define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
# else
#  define G_GNUC_NULL_TERMINATED
# endif
#endif
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

PurplePlugin *quote_plugin = NULL;

static void quote_button_press(GtkWidget *button, PurpleConversation *conv){
	PidginConversation *gtkconv = PIDGIN_CONVERSATION(conv);
	GtkIMHtml *imhtml = GTK_IMHTML(gtkconv->imhtml);
	GtkTextBuffer *buffer = imhtml->text_buffer;
	GtkTextIter start, end;
	char *pBuf = "";
	int count;
	
	if (!gtk_text_buffer_get_selection_bounds(buffer, &start, &end)) {

		gtk_text_buffer_get_end_iter(buffer, &start);
		gtk_text_iter_set_line_offset(&start, 0);
		do {
			count = gtk_text_iter_get_chars_in_line(&start);
			if (count > 0){
				end = start;
				gtk_text_iter_set_line_offset(&end, count - 1);
				pBuf = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);

				count = strlen(pBuf);
				if ( count > 0 ) {
					pBuf = g_strdup_printf("> %s\n", pBuf);
					break;
				}
			}
		} while (gtk_text_iter_backward_line(&start));
	} else {
		int i;
		GtkTextIter iter;
		gchar *quote_text = "> ";
		GtkTextBuffer *tmpbuf = gtk_text_buffer_new(gtk_text_buffer_get_tag_table(buffer));
		
		gtk_text_buffer_get_start_iter(tmpbuf, &iter);
		gtk_text_buffer_insert_range(tmpbuf, &iter, &start, &end);

		count = gtk_text_buffer_get_line_count(tmpbuf);
		for (i = 0; i < count; ++i) {
			gtk_text_iter_set_line(&iter, i);
			gtk_text_buffer_insert(tmpbuf, &iter, quote_text, -1);
		}
		
		gtk_text_buffer_get_bounds (tmpbuf, &start, &end);
		pBuf = g_strdup_printf("%s\n", gtk_text_buffer_get_text(tmpbuf, &start, &end, FALSE));
	}
	
 	gtk_text_buffer_insert_at_cursor(gtkconv->entry_buffer, pBuf, -1);
 	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
	gtk_widget_grab_focus(gtkconv->entry);
	
 	g_free(pBuf);
	
//	purple_debug_error("quote", "line: %s\n", pBuf);
}

static void quote_add_button(PurpleConversation *conv){
	PidginConversation *gtkconv;
	GtkWidget *toolbar, *sep, *quote_button, *label, *bbox, *image;

	gtkconv = PIDGIN_CONVERSATION(conv);
	toolbar = gtkconv->toolbar;

	sep = gtk_vseparator_new();
	g_object_set_data(G_OBJECT(toolbar), PLUGIN_ID "-sep", sep);
	gtk_box_pack_start(GTK_BOX(toolbar), sep, FALSE, FALSE, 0);
	gtk_widget_show_all(sep);

	quote_button = gtk_toggle_button_new();
	gtk_widget_set_name(quote_button, "convquote_button");
	gtk_button_set_relief(GTK_BUTTON(quote_button), GTK_RELIEF_NONE);
	g_object_set_data(G_OBJECT(toolbar), PLUGIN_ID "-quote", quote_button);
	g_signal_connect_swapped(G_OBJECT(quote_button), "button-press-event", G_CALLBACK(gtk_widget_activate), quote_button);
	g_signal_connect(G_OBJECT(quote_button), "activate", G_CALLBACK(quote_button_press), conv);
	
	image = gtk_image_new_from_stock(GTK_STOCK_EDIT, gtk_icon_size_from_name(PIDGIN_ICON_SIZE_TANGO_EXTRA_SMALL));
	label = gtk_label_new_with_mnemonic(_("_Quote"));
	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);

	bbox = gtk_hbox_new(FALSE, 3);
	gtk_container_add(GTK_CONTAINER(quote_button), bbox);
	gtk_box_pack_start(GTK_BOX(bbox), image, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(bbox), label, FALSE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(toolbar), quote_button, FALSE, FALSE, 0);
	gtk_widget_show_all(quote_button);
}

static gboolean plugin_load (PurplePlugin * plugin){
	void *conv_handle = purple_conversations_get_handle();

	purple_signal_connect(conv_handle, "conversation-created", plugin, PURPLE_CALLBACK(quote_add_button), NULL);

	return TRUE;
}

static PurplePluginInfo info = {
	PURPLE_PLUGIN_MAGIC,
	PURPLE_MAJOR_VERSION,
	PURPLE_MINOR_VERSION,
	PURPLE_PLUGIN_STANDARD,
	NULL,
	0,
	NULL,
	PURPLE_PRIORITY_DEFAULT,
	"core-quote_text",
	"Quote text",
	QUOTE_VERSION,
	"Quotes selected message to entry area",
	"Quotes selected message to entry area",
	"Anton Pomozov <pomozoff@gmail.com>",
	"http://launchpad.net/quote/",
	plugin_load,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

static void init_plugin (PurplePlugin * plugin){
}


PURPLE_INIT_PLUGIN (quote_text, init_plugin, info)

